function [Epeak,Wpeak,Irep,Icor,Ierr,a,b,c,d,C,Apeak]=alpa(Iexp,esv,E,isshift,expvar);

%--------------------------------------------------------------
% alpa program by Mojtaba Kooshki and J.M. Daz
% November 2010 
%
% Asymmetric Logistic Peak Adjustment (ALPA)
%
% Adjust the evolution of peak-shaped signals in the experimental matrix
% Iexp. E is the potential vector and expvar is the experimental variable 
% changing every voltammogram (e.g., pH, ligand concentration, etc.).
%
% The program adjusts the peaks to asymmetric logistic function
%
% esv contains the initial estimation of reference pure signals
%
% isshift has 4 rows and as many columns as components; the 1st row
% contains '1' for potential-moving components and '0' for immobile ones;
% the 2nd row contains '1' for peaks changing their width/symmetry and '0'
% for shape-constant peaks; the 3rd and 4th rows define the existence range
% of every component (as experiment numbers).
%
% 
% Irep, Icor are the reproduced and corrected matrices, respectively
%
% C is the concentration profiles matrix
%
% Apeak is the peak area
%
% matlab functions required: alpacalc1,alpacalc2
%
% function [Epeak,Wpeak,Irep,Icor,Ierr,a,b,c,d,C,Apeak]=alpa(Iexp,esv,E,isshift,expvar);
%--------------------------------------------------------------

%checking dimensions
[nexpvar,nothing]=size(expvar);
[nexp,npot]=size(Iexp);
if nexp>npot,Iexp=Iexp';
	disp('There are more voltammograms than potentials. Matrix has been transposed')
	[nexp,npot]=size(Iexp);
end;%(it assumes that usually there are more potentials than voltammograms)
[nri,nci]=size(isshift);
[npot2,ncomp]=size(esv);
[npot3,nothing]=size(E);
exist1=isshift(3,:);%beginning of existence range
exist2=isshift(4,:);%end of existence range
if nargin==6;
	[nexpsel,ncompsel]=size(csel);
	if nexpsel~=nexp, error('csel and Iexp dimensions must agree');end
	if ncompsel~=ncomp, error('csel and esv dimensions must agree');end
end
if npot2~=npot, error('Iexp and esv dimensions must agree');end
if npot3~=npot, error('Iexp and E dimensions must agree');end
if nexpvar~=nexp, error('Iexp and expvar dimensions must agree');end
if nci~=ncomp, error('Isshift and esv dimensions must agree');end
if nri~=4, error('Isshift must have 4 rows');end


%normalized variable number of column
for i=1:npot,x(i,1)=i/npot;end

%normalization of Iexp and esv (Iexpn and v)
maxI=max(max(Iexp));
Iexpn=Iexp./maxI;
maxv=max(max(esv));
v=esv./maxv;
v0=v;%save initial estimation of pure voltammograms as v0 (in prevention of further baseline changes)

% initialization of delta0, delta

delta0=zeros(4,ncomp); delta=zeros(4,ncomp);

% initial estimation of a, b, c, d parameters from esv reference
% voltammogram matrix; they are integrated into delta0 matrix

for j=1:ncomp;

	[ar,xr]=max(v(:,j));
	br=E(xr);
	cr=.01; dr=1;
	
   deltaini0=[ar;br;cr;dr];
   options=optimset('Display','off','Diagnostics','off','MaxIter',100,'TolX',0.00001,'TolFun',0.00001);
   lb=[0;-2;-10;-10];ub=[10;2;100;100];
   [deltaini]=lsqcurvefit('alpacalc1',deltaini0,E,v(:,j),lb,ub,options);
    
	a0(j,:)=deltaini(1,:);
	b0(j,:)=deltaini(2,:);
	c0(j,:)=deltaini(3,:);
	d0(j,:)=deltaini(4,:);

end

delta0=[];
delta0=[a0';b0';c0';d0'];

% Creation of the existence matrix (1 exists, 0 don't exist)%%%%%%%%%%%%%%%

if nargin<6;
    exist=ones(nexp,ncomp);
    for i=1:nexp;
        for k=1:ncomp;
            if exist1(1,k)>i | exist2(1,k)<i;
                exist(i,k)=0;
            end
        end
    end
else exist=csel;
end

% Fitting of peak parameters %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

Ev=[E,zeros(npot,ncomp-1)];

%minE=min(E);maxE=max(E);

figure;
plot(-E,Iexp,'g',-E,esv,'-');title('data matrix vs. pure signal estimation / press any key')
xlabel('-E / V');ylabel('normalised current');
disp('press any key');pause;

for i=1:nexp;
	Esv=[isshift(1:2,:);a0';b0';c0';d0';exist(i,:);v;Ev];
		for k=1:ncomp,%takes advantage of the previously fitted parameters
			if exist(i,k)==1 & i>exist1(1,k),delta0(:,k)=[a(i-1,k);b(i-1,k);c(i-1,k);d(i-1,k)];end
        end
   options=optimset('Display','off','Diagnostics','off','MaxIter',100,'TolX',0.00001,'TolFun',0.00001);
   lb=[0;-2;-10;-10];ub=[10;2;10;100];
   [delta]=lsqcurvefit('alpacalc2',delta0,Esv,Iexpn(i,:),lb,ub,options);
    
	a(i,:)=delta(1,:);
	b(i,:)=delta(2,:);
	c(i,:)=delta(3,:);
	d(i,:)=delta(4,:);
    
	for k=1:ncomp,%forcing again zero concentration in the regions were the peak does not exist
		if exist(i,k)==0, a(i,k)=0;b(i,k)=0;c(i,k)=0;d(i,k)=0; end 
	end
end

% Plot of fitted parameters %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

figure
subplot(4,1,1);plot(expvar,a,'o-');ylabel('a');
title('Fitted variables. Press any key');
subplot(4,1,2);plot(expvar,b,'o-');ylabel('b');
subplot(4,1,3);plot(expvar,c,'o-');ylabel('c');
subplot(4,1,4);plot(expvar,d,'o-');xlabel('experimental variable');ylabel('d');
disp('iterative process finished');pause;

% Corrected (Icor), reproduced (Irep) and error (Ierr) matrices %%%%%%%%%%%
% Lack of fit (lof) between Irep and Iexp

ss1=0;ss2=0;cnonzero=[];
disp(' ');disp('Insert the number of experiment for every reference voltammogram');
vrefnum=input('For instance [1 23 8] ');disp(' ');
b1=b;
c1=c;
d1=d;

for i=1:ncomp 
 b1(:,i)=  b1(vrefnum(i),i);
 c1(:,i)=  c1(vrefnum(i),i);
 d1(:,i)=  d1(vrefnum(i),i);
end

for k=1:nexp;
    deltarep=[];
    deltarep1=[];
    deltarep=[a(k,:);b1(k,:);c1(k,:);d1(k,:);exist(k,:)];
    deltarep1=[a(k,:);b(k,:);c(k,:);d(k,:);exist(k,:)];
    Esvrep1=[isshift(1:2,:);deltarep1;v;Ev];
	 Esvrep=[isshift(1:2,:);deltarep;v;Ev];
    [Irepn(k,:)]=alpacalc2(deltarep1,Esvrep1);
    [Icorn(k,:)]=alpacalc2(deltarep,Esvrep);
	for j=1:npot
    ss1=ss1+(Iexpn(k,j)-Irepn(k,j))^2;
    ss2=ss2+(Iexpn(k,j))^2;
	end
end
Icor=maxI*Icorn';
Irep=Irepn*maxI;
Ierr=(Irepn-Iexpn)*100;
lof=((ss1/ss2)^0.5)*100;
disp('lack of fit (%) :  ');disp(lof);

% Epeak, Wpeak, Apeak %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

Epeak=b;


for i=1:nexp;
    for k=1:ncomp
        if exist(i,k)==1
			   Ipure=[];ipuremax=[];Ipureleft=[];Ipureright=[];ileft=[];iright=[];
            Ipure=(a(i,k)*(1+exp(-(E+c(i,k)*log(d(i,k))-b(i,k))/c(i,k))).^(-d(i,k)-1)).*((d(i,k).^-d(i,k))).*((d(i,k)+1).^(d(i,k)+1)).*(exp(-(E+c(i,k)*log(d(i,k))-b(i,k))/c(i,k)));
				[nothing,ipuremax]=max(Ipure);
				Ipureleft=Ipure(1:ipuremax,1);Ipureright=Ipure(ipuremax+1:end,1);
				[nothing,ileft]=min((Ipureleft-a(i,k)/2).^2);[nothing,iright]=min((Ipureright-a(i,k)/2).^2);
				Wpeak(i,k)=abs(E(ileft,1)-E(ipuremax+iright,1));
				Apeak(i,k)=integrator(abs(E),Ipure,10);
        else Wpeak(i,k)=0;Apeak(i,k)=0;
        end
    end
end

% plot of Epeak, Wpeak and concentration from currents and areas

figure;
subplot(2,2,1);plot(expvar,Epeak,'o');xlabel('Experimental variable');ylabel('Epeak');
axis([min(expvar) max(expvar) min(E) max(E)]);
subplot(2,2,2);plot(expvar,Wpeak,'o');xlabel('Experimental variable');ylabel('W1/2');
axis([min(expvar) max(expvar) 0.01 max(max(Wpeak))+0.02]);
subplot(2,2,3);plot(expvar,a,'o');xlabel('Experimental variable');ylabel('C from Ipeak');
subplot(2,2,4);plot(expvar,Apeak,'o');xlabel('Experimental variable');ylabel('C from Apeak');
disp('press any key');pause

disp('Do you want to estimate concentration profiles') 
iestC=input('from peak currents (1) or from peak areas (2)?');
if iestC==1, C=a;
elseif iestC==2, C=Apeak;
end

% plotting final results%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

figure
subplot(2,2,1);plot(E,Iexp);title('experimental matrix');ylabel('current / A');xlabel('E / V');
subplot(2,2,2);plot(E,Ierr);title('error matrix');ylabel('% error');xlabel('E / V');
subplot(2,2,3);plot(E,Irep);title('reproduced matrix');ylabel('current / A');xlabel('E / V');
subplot(2,2,4);plot(E,Icor);title('corrected matrix');ylabel('current / A');xlabel('E / V');
